@@ -505,11 +505,14 @@ _vte_xft_set_background_image (struct _vte_draw *draw,
 	if (data->pixmap != NULL) {
 		g_object_unref (data->pixmap);
 	}
+	draw->has_background_image = FALSE;
 	data->pixmap = NULL;
 	if (pixmap != NULL) {
 		data->pixmap = pixmap;
 		data->xpixmap = gdk_x11_drawable_get_xid (pixmap);
 		gdk_drawable_get_size (pixmap, &data->pixmapw, &data->pixmaph);
+		draw->has_background_image =
+			data->pixmapw > 0 && data->pixmaph > 0;
 	}
 }
 
